pub struct StableVec<T> { /* private fields */ }Expand description
A Vec<T>-like collection that maintains stable indices for its elements.
Unlike with a standard Vec<T>, removing elements from a StableVec<T> is O(1),
and it does not move other elements or invalidate their indices.
This is achieved by internally storing each element as an Option<T>,
and reusing freed slots for new elements.
Overiew of Important Methods
pushadds a new element and returns its stable index (O(1)).removeremoves an element at a given index and returns it (O(1)).try_removeattempts to remove an element at a given index, returningNoneif it doesn’t exist (O(1)).getandget_mutprovide access to elements by index (O(1)).clearremoves all elements without deallocating memory (O(1)).
Implementations
Sourceimpl<T> StableVec<T>
impl<T> StableVec<T>
Sourcepub fn with_capacity(capacity: usize) -> Self
pub fn with_capacity(capacity: usize) -> Self
Creates a new StableVec with the given initial capacity.
This is useful for preallocating space to avoid reallocations.
Sourcepub fn push(&mut self, element: T) -> usize
pub fn push(&mut self, element: T) -> usize
Pushes a new element to the first available slot, returning its index.
This may reuse a previously freed slot.
Time Complexity
O(1)
Sourcepub fn next_push_index(&self) -> usize
pub fn next_push_index(&self) -> usize
Returns the next index that will be used for a push.
This index may reuse a previously freed slot.
Time Complexity
O(1)
Sourcepub fn try_remove(&mut self, index: usize) -> Option<T>
pub fn try_remove(&mut self, index: usize) -> Option<T>
Sourcepub unsafe fn get_unchecked(&self, index: usize) -> &T
pub unsafe fn get_unchecked(&self, index: usize) -> &T
Sourcepub unsafe fn get_unchecked_mut(&mut self, index: usize) -> &mut T
pub unsafe fn get_unchecked_mut(&mut self, index: usize) -> &mut T
Sourcepub fn get_disjoint_mut2(
&mut self,
index1: usize,
index2: usize,
) -> Option<[&mut T; 2]>
pub fn get_disjoint_mut2( &mut self, index1: usize, index2: usize, ) -> Option<[&mut T; 2]>
Returns mutable references to two disjoint elements at the given indices, if they exist.
If the indices are the same, or if either index is out of bounds or has no element,
None is returned.
Time Complexity
O(1)
Sourcepub unsafe fn get_disjoint_mut_unchecked(
&mut self,
index1: usize,
index2: usize,
) -> [&mut T; 2]
pub unsafe fn get_disjoint_mut_unchecked( &mut self, index1: usize, index2: usize, ) -> [&mut T; 2]
Returns mutable references to two disjoint elements at the given indices without bounds checking.
If the indices are the same, or if either index has no element, the behavior is undefined.
Time Complexity
O(1)
Safety
The caller must ensure that the indices are disjoint and that there are elements at both indices.